import React, { useEffect } from 'react';
import { memoize } from 'react-context-hooks-store';
import { NonIdealState, Spinner } from '@blueprintjs/core';
import HTML5Backend from 'react-dnd-html5-backend'
import { DndProvider, DragSource, DropTarget } from 'react-dnd';
import BpTable from '../BpTable';

import './index.less';

const dragDirection = (dragIndex, hoverIndex, initialClientOffset, clientOffset, sourceClientOffset) => {
  const hoverMiddleY = (initialClientOffset.y - sourceClientOffset.y) / 2;
  const hoverClientY = clientOffset.y - sourceClientOffset.y;
  if (dragIndex < hoverIndex && hoverClientY > hoverMiddleY) {
    return 'downward';
  }
  if (dragIndex > hoverIndex && hoverClientY < hoverMiddleY) {
    return 'upward';
  }
};

let BodyRow = (props) => {
  const {
    isOver,
    connectDragSource,
    connectDropTarget,
    moveRow,
    dragRow,
    clientOffset,
    sourceClientOffset,
    initialClientOffset,
    ...restProps
  } = props;
  const style = { ...restProps.style, cursor: 'move' };

  let className = restProps.className;
  if (isOver && initialClientOffset) {
    const direction = dragDirection(
      dragRow.index,
      restProps.index,
      initialClientOffset,
      clientOffset,
      sourceClientOffset
    );
    if (direction === 'downward') {
      className += ' app-drop-over-downward';
    }
    if (direction === 'upward') {
      className += ' app-drop-over-upward';
    }
  }

  return connectDragSource(
    connectDropTarget(
      <tr
        {...restProps}
        className={className}
        style={style}
      />
    )
  );
};

const rowSource = {
  beginDrag(props) {
    return {
      index: props.index
    };
  },
};

const rowTarget = {
  drop(props, monitor) {
    const dragIndex = monitor.getItem().index;
    const hoverIndex = props.index;
    if (dragIndex === hoverIndex) {
      return;
    }
    props.moveRow(dragIndex, hoverIndex);
    monitor.getItem().index = hoverIndex;
  },
};

BodyRow = DropTarget('row', rowTarget, (connect, monitor) => ({
  connectDropTarget: connect.dropTarget(),
  isOver: monitor.isOver(),
  sourceClientOffset: monitor.getSourceClientOffset(),
}))(
  DragSource('row', rowSource, (connect, monitor) => ({
    connectDragSource: connect.dragSource(),
    dragRow: monitor.getItem(),
    clientOffset: monitor.getClientOffset(),
    initialClientOffset: monitor.getInitialClientOffset(),
  }))(BodyRow)
);

function List(props) {
  const { fieldList = [], dataList = [], isDrag, dragName, isLoading, ...tableProps } = props;

  useEffect(() => {
    //event
  }, [fieldList, dataList]);

  const moveRow = (dragIndex, hoverIndex) => {
    const dragRow = dataList[dragIndex];
    const hoverRow = dataList[hoverIndex];

    function updateRank() {
      //update
    }
  };

  if (isDrag) {
    tableProps.onRow = (record, index) => ({
      index,
      moveRow
    });

    tableProps.components = {
      body: {
        row: BodyRow
      }
    };
  }

  // tableProps.onRow = props.onRow;

  // const isLoading = true;

  return (
    <>
      {
        isLoading
        &&
        <Spinner intent={'primary'} size={30}/>
      }
      {
        !isLoading
        &&
        <DndProvider backend={HTML5Backend}>
          <BpTable
            bordered={true}
            columns={fieldList}
            data={dataList}
            emptyText={(
              <NonIdealState
                icon="th"
                title="Chưa có dữ liệu"
              />
            )}
            {...tableProps}
          />
        </DndProvider>
      }
    </>
  )
}

// const memoProps = (prevProps, nextProps) => {
//   if (JSON.stringify(prevProps.fieldList) !== JSON.stringify(nextProps.fieldList)) {
//     return false;
//   }
//
//   if (JSON.stringify(prevProps.dataList) !== JSON.stringify(nextProps.dataList)) {
//     return false;
//   }
//
//   if (prevProps.isLoading !== nextProps.isLoading) {
//     return false;
//   }
//
//   return true;
// };

export default memoize(List);
